"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var nanoid = require("nanoid/async");
var unix_dgram_socket_1 = require("unix-dgram-socket");
var events_1 = require("events");
var IpcNetworkError_1 = require("./IpcNetworkError");
var IpcNetwork = (function (_super) {
    tslib_1.__extends(IpcNetwork, _super);
    function IpcNetwork(processName, rpcCallback) {
        var _this = _super.call(this) || this;
        _this.rpcCallback = rpcCallback;
        _this.jobsItems = {};
        _this.processName = (processName) ? processName : "pid-" + process.ppid + "-" + process.pid;
        _this.unixSocketPath = IpcNetwork.getSocketPathFromName(_this.processName);
        _this.unixSocket = new unix_dgram_socket_1.UnixDgramSocket();
        _this.unixSocket.on('error', function (error) {
            _this.emit('error', error);
        });
        _this.unixSocket.on('message', function (message, info) {
            var remoteSocketName = IpcNetwork.getSocketNameFromPath(info.remoteSocket);
            var messageType = message.readUInt8(0);
            if (messageType === IpcNetwork.TYPE_MESSAGE) {
                _this.emit('message', {
                    message: message.slice(1),
                    from: remoteSocketName,
                });
            }
            else if (messageType === IpcNetwork.TYPE_RPC_JOB) {
                var jobId = message.slice(1, IpcNetwork.JOB_ID_LENGTH + 1).toString(unix_dgram_socket_1.UnixDgramSocket.payloadEncoding);
                var jobName = message.slice(1 + IpcNetwork.JOB_ID_LENGTH).toString(unix_dgram_socket_1.UnixDgramSocket.payloadEncoding);
                var socketPath = IpcNetwork.getSocketPathFromName(remoteSocketName);
                var sendResult = false;
                try {
                    var jobResult = _this.rpcCallback(jobName, remoteSocketName);
                    sendResult = _this.unixSocket.send(IpcNetwork.composeMessage(IpcNetwork.TYPE_RPC_RESULT, jobResult, jobId), socketPath);
                }
                catch (error) {
                    sendResult = _this.unixSocket.send(IpcNetwork.composeMessage(IpcNetwork.TYPE_RPC_ERROR, error.message, jobId), socketPath);
                }
                if (!sendResult) {
                    _this.emit('error', "Unable to send job results to remote process: " + remoteSocketName);
                }
            }
            else if (messageType === IpcNetwork.TYPE_RPC_RESULT || messageType === IpcNetwork.TYPE_RPC_ERROR) {
                var jobId = message.slice(1, IpcNetwork.JOB_ID_LENGTH + 1).toString(unix_dgram_socket_1.UnixDgramSocket.payloadEncoding);
                if (_this.jobsItems[jobId]) {
                    if (_this.jobsItems[jobId].timeoutTimer) {
                        clearTimeout(_this.jobsItems[jobId].timeoutTimer);
                    }
                    if (messageType === IpcNetwork.TYPE_RPC_ERROR) {
                        var errorMessage = message.slice(1 + IpcNetwork.JOB_ID_LENGTH).toString(unix_dgram_socket_1.UnixDgramSocket.payloadEncoding);
                        _this.jobsItems[jobId].reject(new IpcNetworkError_1.IpcNetworkError(IpcNetwork.ERROR_RPC_EXEC, errorMessage));
                    }
                    else {
                        _this.jobsItems[jobId].resolve(message.slice(1 + IpcNetwork.JOB_ID_LENGTH));
                    }
                    delete _this.jobsItems[jobId];
                }
            }
            else {
                _this.emit('error', new Error("Received unknown message type from: " + remoteSocketName));
            }
        });
        return _this;
    }
    IpcNetwork.prototype.startListening = function () {
        this.unixSocket.bind(this.unixSocketPath);
    };
    IpcNetwork.prototype.stopListening = function () {
        this.unixSocket.close();
    };
    IpcNetwork.prototype.send = function (data, socketName) {
        var socketPath = IpcNetwork.getSocketPathFromName(socketName);
        return this.unixSocket.send(IpcNetwork.composeMessage(IpcNetwork.TYPE_MESSAGE, data), socketPath);
    };
    IpcNetwork.prototype.sendRpc = function (jobName, socketName, timeout) {
        var _this = this;
        return new Promise(function (resolve, reject) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var jobId, socketPath, message, timeoutTimer, result;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, nanoid(IpcNetwork.JOB_ID_LENGTH)];
                    case 1:
                        jobId = _a.sent();
                        socketPath = IpcNetwork.getSocketPathFromName(socketName);
                        message = IpcNetwork.composeMessage(IpcNetwork.TYPE_RPC_JOB, jobName, jobId);
                        timeoutTimer = null;
                        if (timeout) {
                            timeoutTimer = setTimeout(function () {
                                reject(new IpcNetworkError_1.IpcNetworkError(IpcNetwork.ERROR_RPC_TIMEOUT, "Job timeout (> " + timeout + "ms)"));
                                delete _this.jobsItems[jobId];
                            }, timeout);
                        }
                        this.jobsItems[jobId] = { resolve: resolve, reject: reject, timeoutTimer: timeoutTimer };
                        result = this.unixSocket.send(message, socketPath);
                        if (!result) {
                            delete this.jobsItems[jobId];
                            reject(new IpcNetworkError_1.IpcNetworkError(IpcNetwork.ERROR_RPC_SEND, "Unable to send job to remote process: " + socketPath));
                        }
                        return [2];
                }
            });
        }); });
    };
    IpcNetwork.composeMessage = function (messageType, messageData, jobId) {
        var resultBuffer = [];
        var type = Buffer.allocUnsafe(1);
        type.writeUInt8(messageType, 0);
        resultBuffer.push(type);
        if (jobId) {
            resultBuffer.push(Buffer.from(jobId, unix_dgram_socket_1.UnixDgramSocket.payloadEncoding));
        }
        if (typeof messageData === 'string') {
            resultBuffer.push(Buffer.from(messageData, unix_dgram_socket_1.UnixDgramSocket.payloadEncoding));
        }
        return Buffer.concat(resultBuffer);
    };
    IpcNetwork.getSocketPathFromName = function (socketName) {
        return "@/tmp/nodejs/.internal/ipc-network/nodes/" + socketName;
    };
    IpcNetwork.getSocketNameFromPath = function (socketPath) {
        return socketPath.replace(IpcNetwork.getSocketPathFromName(''), '');
    };
    IpcNetwork.ERROR_RPC_SEND = 'rpc_send_error';
    IpcNetwork.ERROR_RPC_TIMEOUT = 'rpc_timeout_error';
    IpcNetwork.ERROR_RPC_EXEC = 'rpc_exec_error';
    IpcNetwork.TYPE_MESSAGE = 0x01;
    IpcNetwork.TYPE_RPC_JOB = 0xA0;
    IpcNetwork.TYPE_RPC_RESULT = 0xA1;
    IpcNetwork.TYPE_RPC_ERROR = 0xAA;
    IpcNetwork.JOB_ID_LENGTH = 25;
    return IpcNetwork;
}(events_1.EventEmitter));
exports.IpcNetwork = IpcNetwork;
//# sourceMappingURL=IpcNetwork.js.map